//********************** x16demo.c ****************************
//********************** Windows CE ***************************
//* This file contains demo (test) program for Sensoray model
//* 516/616 MPEG codec.
//* Written by Alexander Kostromitin
//* Copyright (C) Sensoray Company 2001
//*************************************************************

#include <time.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <commdlg.h>    // includes common dialog functionality
#include <commctrl.h>
#include <dlgs.h>       // includes common dialog template defines

#define MAXPATH 256

#include "x16demo.h"
#include "..\x16dll\sm2288.h"
#include "sm2288f.h"

int APIENTRY MainWndProc(HWND hDlg, int uMsg, WPARAM wParam, LPARAM lParam);
BOOL InitApplication(void);
BOOL InitInstance(void);
BOOL CALLBACK startdlg(HWND hDlg, UINT message, UINT wParam, LONG lParam);

void enablecontrols(int enkey);
void setboard(int board);
void readwrite(void);

int getfilename(wchar_t *name, wchar_t *title);
int error(wchar_t *hdr, int key, wchar_t *fmt, ... );
time_t time_(time_t *t);

enum modes { STOP, PAUSE, PLAY, WRITE };

HINSTANCE MainInst;             // the current instance
HWND      MainWin;              // hWnd of main window

wchar_t *AppName = TEXT( "x16demo" );
wchar_t *AppHdr = TEXT( "x16demo" );

int     num_cards,              // number of x16 cards
        card;                   // current card
int     insys,                  // input video system PAL/NTSC
        outsys,                 // output video system PAL/NTSC
        br,                     // bitrate
        vbr,                    // vbr flag
        sz,                     // size of MPEG image
        innum,                  // video input
        intype,                 // video type C-video/S-video
        pause,                  // pause flag
        bright,                 // brightness
        contr,                  // contrast
        satur,                  // saturation
        bars,                   // color bars flag
        filter,                 // input video filter flag
        volume,                 // audio volume
        mute;                   // audio mute flag

enum modes mode;                // current mode
wchar_t fname[MAXPATH] = TEXT("\\hard disk\\tmp.mpg"); // name of MPEG
time_t starttime, pausetime, rtime;
int rlen;
int run, endkey;
HANDLE file;
char iobuffer[1024 * 32];       // I/O SM buffer; maximum buffer size is 32k

wchar_t *mnames[] = {  TEXT( "STOP" ),  TEXT( "PAUSE" ),  TEXT( "PLAY" ),  TEXT( "WRITE" ) };

HANDLE thandle;
DWORD threadID;



int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nCmdShow)
{
  int err;

  err = X16_DLLOpen();          // open DLL; see sx16app.c
  if(err)
    {
    error(AppName, MB_ICONERROR,  TEXT( "DLL open error #%d" ), err);
    return(0);
    }
  num_cards = X16_open();
  if(num_cards < 0)
    {
    error(AppName, MB_ICONERROR, TEXT( "Driver open error %d" ), -num_cards);
    return(0);                  // problem with driver
    }
  if(num_cards == 0)
    {
    error(AppName, MB_ICONERROR, TEXT( "No x16 devices found" ));
    return(0);                  // no x16 devices or incorrect 516 I/O port selected
    }
  X16_register(card = 0);       // register card
  MainInst = hInstance;
  if(!InitApplication()) return(0);
  if(!InitInstance()) return(0);
  X16_unregister(card);         // unregister card
  X16_close();                  // close device
  X16_DLLClose();               // close DLL
  return(0);
}


BOOL InitApplication(void)
{
  WNDCLASS wc;

  insys = outsys = NTSC; br = 1000000; vbr = 0; sz = SIZE352_2; mode = STOP; intype = CVIDEO; innum = 0;
  volume = 6;  // -6 dB
  bright = contr = satur = 128;

  ZeroMemory ( &wc, sizeof( wc ) );
  wc.style = 0;
  wc.lpfnWndProc = (WNDPROC)MainWndProc;
  wc.cbClsExtra = 0;
  wc.cbWndExtra = 0;
  wc.hInstance = MainInst;
  //wc.hIcon = LoadIcon(MainInst, TEXT( "MainIcon" ));
  wc.hCursor = 0;
  wc.hbrBackground = 0;
  wc.lpszMenuName = 0;
  wc.lpszClassName = AppName;
  return(RegisterClass(&wc));
}


BOOL InitInstance(void)
{
  static int err;
  MainWin = CreateWindow(
      AppName,
      AppHdr,
      0,
      CW_USEDEFAULT,
      CW_USEDEFAULT,
      100,
      100,
      NULL,
      NULL,
      MainInst,
      NULL);
  if(!MainWin) return(FALSE);
  ShowWindow(MainWin, SW_MINIMIZE);
  return(TRUE);
}


int APIENTRY MainWndProc(HWND hDlg, int uMsg, WPARAM wParam, LPARAM lParam)
{
  switch(uMsg)
    {
    case WM_CREATE:
      DialogBox(MainInst, TEXT( "Edit" ), MainWin, startdlg);
      break;
    case WM_DESTROY:
      PostQuitMessage(0);
      break;
    default: return(DefWindowProc(hDlg, uMsg, wParam, lParam));
    }
  return(0);
}


BOOL CALLBACK startdlg(HWND hDlg, UINT message, UINT wParam, LONG lParam)
{
  int newmode;
  BOOL ok;
  int tmp;

  switch(message)
    {
    case WM_INITDIALOG:
      MainWin = hDlg;
      SetTimer(hDlg, 1, 500, 0);

      // GPIO test
        {
        static int c;

        X16_write_gpio(card, 0);
        c = X16_read_gpio(card);
        X16_write_gpio(card, 3);
        c = X16_read_gpio(card);
        }
      // set up default control's states and x16 values
      SetWindowPos(MainWin, HWND_TOPMOST, 50, 50, 0, 0, SWP_NOSIZE);
      SetWindowPos(MainWin, HWND_NOTOPMOST, 50, 50, 0, 0, SWP_NOSIZE);
      SendMessage(hDlg, WM_SETICON, ICON_SMALL, (LPARAM)LoadIcon(MainInst,  TEXT( "MainIcon" )));

      SetDlgItemText(hDlg, IDC_FILE, fname);
      SetDlgItemText(hDlg, IDC_MODE, mnames[mode]);
      SetDlgItemInt(hDlg, IDC_RATE, br, 0);
      SetDlgItemInt(hDlg, IDC_TDISP, rtime, 0);
      SetDlgItemInt(hDlg, IDC_LDISP, rlen, 0);
      SendDlgItemMessage(hDlg, IDC_NTSC_IN, BM_SETCHECK, insys == NTSC, 0);
      SendDlgItemMessage(hDlg, IDC_PAL_IN, BM_SETCHECK, insys == PAL, 0);
      SendDlgItemMessage(hDlg, IDC_NTSC_OUT, BM_SETCHECK, outsys == NTSC, 0);
      SendDlgItemMessage(hDlg, IDC_PAL_OUT, BM_SETCHECK, outsys == PAL, 0);

      SendDlgItemMessage(hDlg, IDC_COMPOSITE, BM_SETCHECK, intype == CVIDEO, 0);
      SendDlgItemMessage(hDlg, IDC_SVIDEO, BM_SETCHECK, intype == SVIDEO, 0);

      SendDlgItemMessage(hDlg, IDC_IN1, BM_SETCHECK, innum == 0, 0);
      SendDlgItemMessage(hDlg, IDC_IN2, BM_SETCHECK, innum == 1, 0);
      SendDlgItemMessage(hDlg, IDC_IN3, BM_SETCHECK, innum == 2, 0);
      SendDlgItemMessage(hDlg, IDC_IN4, BM_SETCHECK, innum == 3, 0);

      SendDlgItemMessage(hDlg, IDC_SIZE176_1, BM_SETCHECK, sz == SIZE176_1, 0);
      SendDlgItemMessage(hDlg, IDC_SIZE352_1, BM_SETCHECK, sz == SIZE352_1, 0);
      SendDlgItemMessage(hDlg, IDC_SIZE352_2, BM_SETCHECK, sz == SIZE352_2, 0);
      SendDlgItemMessage(hDlg, IDC_SIZE480_2, BM_SETCHECK, sz == SIZE480_2, 0);
      SendDlgItemMessage(hDlg, IDC_SIZE640_2, BM_SETCHECK, sz == SIZE640_2, 0);
      SendDlgItemMessage(hDlg, IDC_SIZE704_2, BM_SETCHECK, sz == SIZE704_2, 0);
      SendDlgItemMessage(hDlg, IDC_SIZE720_2, BM_SETCHECK, sz == SIZE720_2, 0);

      SendDlgItemMessage(hDlg, IDC_VBR, BM_SETCHECK, vbr, 0);
      SendDlgItemMessage(hDlg, IDC_BARS, BM_SETCHECK, bars, 0);
      SendDlgItemMessage(hDlg, IDC_FILTER, BM_SETCHECK, filter, 0);

      SetDlgItemInt(hDlg, IDC_BRGHT, bright, 0);
      SetDlgItemInt(hDlg, IDC_CONTR, contr, 0);
      SetDlgItemInt(hDlg, IDC_SAT, satur, 0);

      SetDlgItemInt(hDlg, IDC_VOL, volume, 0);

      X16_set_brightness(card, bright);
      X16_set_contrast(card, contr);
      X16_set_saturation(card, satur);
      X16_set_volume(card, volume);

      SendDlgItemMessage(hDlg, IDC_MUTE, BM_SETCHECK, mute, 0);
      X16_set_mute(card, mute);

      X16_set_input_system(card, insys);
      X16_set_output_system(card, outsys);
      X16_set_input(card, innum);
      X16_set_input_type(card, intype);

      // create thread for i/o control
      if((thandle = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)readwrite,
          NULL, 0, &threadID)) == 0) return(FALSE);
      SetThreadPriority(thandle,  THREAD_PRIORITY_HIGHEST);
      break;
    case WM_DESTROY:
      break;
    // process commands
    case WM_COMMAND:
      switch(LOWORD(wParam))
        {
        case IDC_BROWSE:
          GetDlgItemText(hDlg, IDC_FILE, fname, MAXPATH - 1);
          if(getfilename(fname, TEXT( "Open file" )) == 0) break;
          SetDlgItemText(hDlg, IDC_FILE, fname);
          break;
        case IDC_BARS:
          tmp = SendDlgItemMessage(hDlg, IDC_BARS, BM_GETCHECK, 0, 0);
          if(tmp != bars) X16_set_colorbars(card, bars = tmp);
          break;
        case IDC_FILTER:
          tmp = SendDlgItemMessage(hDlg, IDC_FILTER, BM_GETCHECK, 0, 0);
          if(tmp != filter) X16_set_filter(card, filter = tmp);
          break;
        case IDC_MUTE:
          tmp = SendDlgItemMessage(hDlg, IDC_MUTE, BM_GETCHECK, 0, 0);
          if(tmp != mute)
            {
            X16_set_mute(card, mute = tmp);
            }
          break;
        case IDC_NTSC_IN:
        case IDC_PAL_IN:
          if(mode != STOP) break;
          if(SendDlgItemMessage(hDlg, IDC_NTSC_IN, BM_GETCHECK, 0, 0)) tmp = NTSC;
          else tmp = PAL;
          if(tmp != insys)
            {
            X16_set_input_system(card, insys = tmp);
            }
          break;
        case IDC_NTSC_OUT:
        case IDC_PAL_OUT:
          if(mode != STOP) break;
          if(SendDlgItemMessage(hDlg, IDC_NTSC_OUT, BM_GETCHECK, 0, 0)) tmp = NTSC;
          else tmp = PAL;
          if(tmp != outsys)
            {
            X16_set_output_system(card, outsys = tmp);
            setboard(card);
            }
          break;
        case IDC_COMPOSITE:
        case IDC_SVIDEO:
          if(SendDlgItemMessage(hDlg, IDC_COMPOSITE, BM_GETCHECK, 0, 0)) tmp = CVIDEO;
          else tmp = SVIDEO;
          if(tmp != intype) X16_set_input_type(card, intype = tmp);
          break;
        case IDC_IN1:
        case IDC_IN2:
        case IDC_IN3:
        case IDC_IN4:
          if(SendDlgItemMessage(hDlg, IDC_IN1, BM_GETCHECK, 0, 0)) tmp = 0;
          else if(SendDlgItemMessage(hDlg, IDC_IN2, BM_GETCHECK, 0, 0)) tmp = 1;
          else if(SendDlgItemMessage(hDlg, IDC_IN3, BM_GETCHECK, 0, 0)) tmp = 2;
          else tmp = 3;
          if(innum != tmp) X16_set_input(card, innum = tmp);
          break;
        case IDC_ASET:
          tmp = GetDlgItemInt(hDlg, IDC_VOL, &ok, 0);
          if(!ok || tmp < 0 || tmp > 255) return(FALSE);
          volume = tmp;
          X16_set_volume(card, volume);
          break;
        case IDC_VSET:
          tmp = GetDlgItemInt(hDlg, IDC_BRGHT, &ok, 0);
          if(!ok || tmp < 0 || tmp > 255) return(FALSE);
          bright = tmp;
          tmp = GetDlgItemInt(hDlg, IDC_CONTR, &ok, 0);
          if(!ok || tmp < 0 || tmp > 255) return(FALSE);
          contr = tmp;
          tmp = GetDlgItemInt(hDlg, IDC_SAT, &ok, 0);
          if(!ok || tmp < 0 || tmp > 255) return(FALSE);
          satur = tmp;
          X16_set_brightness(card, bright);
          X16_set_contrast(card, contr);
          X16_set_saturation(card, satur);
          break;
        case IDC_PAUSE:
          if(mode == STOP) break;
          pause = !pause;
          if(pause) SetDlgItemText(hDlg, IDC_MODE,  TEXT( "PAUSE" ));
          else SetDlgItemText(hDlg, IDC_MODE, mnames[mode]);
          break;
        case IDC_PLAY:
          newmode = PLAY; goto rwstart;
        case IDC_WRITE:
          newmode = WRITE;
          rwstart:
          if(mode == newmode) return(FALSE);
          GetDlgItemText(hDlg, IDC_FILE, fname, MAXPATH - 1);
          tmp = GetDlgItemInt(hDlg, IDC_RATE, &ok, 0);
          if(!ok || tmp <= 0 || tmp > 20000000) return(FALSE);
          br = tmp;
          if(SendDlgItemMessage(hDlg, IDC_NTSC_IN, BM_GETCHECK, 0, 0)) insys = NTSC;
          else insys = PAL;
          if(SendDlgItemMessage(hDlg, IDC_NTSC_OUT, BM_GETCHECK, 0, 0)) outsys = NTSC;
          else outsys = PAL;

          if(SendDlgItemMessage(hDlg, IDC_COMPOSITE, BM_GETCHECK, 0, 0)) intype = CVIDEO;
          else intype = SVIDEO;

          if(SendDlgItemMessage(hDlg, IDC_IN1, BM_GETCHECK, 0, 0)) innum = 0;
          else if(SendDlgItemMessage(hDlg, IDC_IN2, BM_GETCHECK, 0, 0)) innum = 1;
          else if(SendDlgItemMessage(hDlg, IDC_IN3, BM_GETCHECK, 0, 0)) innum = 2;
          else innum = 3;

          if(SendDlgItemMessage(hDlg, IDC_SIZE176_1, BM_GETCHECK, 0, 0)) sz = SIZE176_1;
          else if(SendDlgItemMessage(hDlg, IDC_SIZE352_1, BM_GETCHECK, 0, 0)) sz = SIZE352_1;
          else if(SendDlgItemMessage(hDlg, IDC_SIZE352_2, BM_GETCHECK, 0, 0)) sz = SIZE352_2;
          else if(SendDlgItemMessage(hDlg, IDC_SIZE480_2, BM_GETCHECK, 0, 0)) sz = SIZE480_2;
          else if(SendDlgItemMessage(hDlg, IDC_SIZE640_2, BM_GETCHECK, 0, 0)) sz = SIZE640_2;
          else if(SendDlgItemMessage(hDlg, IDC_SIZE704_2, BM_GETCHECK, 0, 0)) sz = SIZE704_2;
          else sz = SIZE720_2;

          vbr = SendDlgItemMessage(hDlg, IDC_VBR, BM_GETCHECK, 0, 0);
          bars = SendDlgItemMessage(hDlg, IDC_BARS, BM_GETCHECK, 0, 0);
          filter = SendDlgItemMessage(hDlg, IDC_FILTER, BM_GETCHECK, 0, 0);

          mute = SendDlgItemMessage(hDlg, IDC_MUTE, BM_GETCHECK, 0, 0);
          if(newmode == WRITE)          // open file to write
            {
            if((file = CreateFile(fname, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_FLAG_WRITE_THROUGH, 0)) == INVALID_HANDLE_VALUE )
              {
              error(AppName, MB_ICONERROR,
                 TEXT( "Open error of file %s" ), fname);
              return(FALSE);
              }
            }
          else                          // open file to read
            {
            if((file = CreateFile(fname, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, 0)) == INVALID_HANDLE_VALUE )
              {
              error(AppName, MB_ICONERROR,
                 TEXT( "File %s not found" ), fname);
              return(FALSE);
              }
            }
          enablecontrols(0);
          // basic parameters, set them first
          X16_set_input_system(card, insys);   // set input video system
          X16_set_output_system(card, outsys); // set output video system
          // use this sequence to modify settings
          X16_set_input_type(card, intype);    // set input type
          X16_set_input(card, innum);          // set input number
          X16_set_colorbars(card, bars);       // set color bars on/off
          X16_set_filter(card, filter);        // set video input filter on/off
          // use in any order

          X16_set_m(card, 4);
          X16_set_n(card, 1);

          X16_set_bitrate(card, br);           // set bitrate
          X16_set_picsize(card, sz);           // set picture format and size
          X16_set_vbr(card, vbr);              // set var. bitrate
          X16_set_volume(card, volume);        // set volume
          X16_set_mute(card, mute);            // set mute

          run = rlen = pause = pausetime = 0;  // clear all
          SetDlgItemText(hDlg, IDC_MODE, mnames[mode = newmode]);
          break;
        case IDC_STOP:
          pause = 0;
          mode = STOP;
          break;
        case IDOK:
        case IDCANCEL:
          mode = STOP;
          endkey = 1;
          KillTimer(hDlg, 1);
          X16_set_mute(card, 1);
          WaitForSingleObject(thandle, 3000);           // wait for thread completed
          if(file) { CloseHandle(file); file = 0; }     // close file
          PostQuitMessage(0);
          break;
        default: return(FALSE);
        }
      break;
    case WM_TIMER:
      {
      wchar_t sbuf[32];
      time_t t;

      if((mode == WRITE || mode == PLAY) && !pause)
        {
        if(run)
          {
          swprintf(sbuf, TEXT( "%d.%03d.%03d" ), rlen / 1000000, rlen / 1000 % 1000, rlen % 1000);
          SetDlgItemText(MainWin, IDC_LDISP, sbuf);
          t=time_(0) - starttime - pausetime;
          if(t!=rtime)
            {
            int rt = t - 2;
            if(rt > 0) SetDlgItemInt(MainWin, IDC_RDISP, rlen / rt, 0);
            SetDlgItemInt(MainWin, IDC_TDISP, rtime = t, 0);
            }
          }
        else time_(&starttime);
        }
      break;
      }
    default: return(FALSE);
    }
  return(TRUE);
}


void enablecontrols(int enkey)  // disable some controls during rec/play
{
  EnableWindow(GetDlgItem(MainWin, IDC_FILE), enkey);
  EnableWindow(GetDlgItem(MainWin, IDC_MODE), enkey);
  EnableWindow(GetDlgItem(MainWin, IDC_RATE), enkey);
  EnableWindow(GetDlgItem(MainWin, IDC_NTSC_IN), enkey);
  EnableWindow(GetDlgItem(MainWin, IDC_PAL_IN), enkey);
  EnableWindow(GetDlgItem(MainWin, IDC_NTSC_OUT), enkey);
  EnableWindow(GetDlgItem(MainWin, IDC_PAL_OUT), enkey);
  EnableWindow(GetDlgItem(MainWin, IDC_COMPOSITE), enkey);
  EnableWindow(GetDlgItem(MainWin, IDC_SVIDEO), enkey);
  EnableWindow(GetDlgItem(MainWin, IDC_SIZE176_1), enkey);
  EnableWindow(GetDlgItem(MainWin, IDC_SIZE352_1), enkey);
  EnableWindow(GetDlgItem(MainWin, IDC_SIZE352_2), enkey);
  EnableWindow(GetDlgItem(MainWin, IDC_SIZE480_2), enkey);
  EnableWindow(GetDlgItem(MainWin, IDC_SIZE640_2), enkey);
  EnableWindow(GetDlgItem(MainWin, IDC_SIZE704_2), enkey);
  EnableWindow(GetDlgItem(MainWin, IDC_SIZE720_2), enkey);
  EnableWindow(GetDlgItem(MainWin, IDC_VBR), enkey);
}


void setboard(int board)                        // change/reset board
{
  mode = STOP;
  endkey = 1;                                   // break readwrite loop
  X16_stop(card);
  X16_set_mute(card, 1);
  WaitForSingleObject(thandle, 3000);           // wait for thread completed
  Sleep(100);
  if(file) { CloseHandle(file); file = 0; }     // close file
  X16_unregister(card);
  card = board;                                 // new board
  X16_register(card);
  endkey = 0;
  X16_set_input_system(card, insys);
  X16_set_output_system(card, outsys);
  X16_set_input_type(card, intype);
  X16_set_input(card, innum);
  X16_set_colorbars(card, bars);
  X16_set_filter(card, filter);
  X16_set_volume(card, volume);
  X16_set_mute(card, mute);
  thandle = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)readwrite, NULL, 0, &threadID);
}


void readwrite(void)                            // rec/play in separated thread
{
  int oldmode = 0, oldp = 0, err = 0, len;
  time_t pt;
  DWORD rwnum;

  while(!endkey)
    {
    if(pause)
      {
      if(!oldp) { X16_pause(card); oldp = 1; time_(&pt); }
      }
    else
      {
      if(oldp) { X16_resume(card); oldp = 0; pausetime += time_(0) - pt; }
      }
    if(oldmode == mode)
      {
      if(mode != PLAY && mode != WRITE || pause)
        { Sleep(50); continue; }                        // no actions
      }
    if(mode == WRITE)                                   // WRITE mode
      {
      len = X16_read(card, iobuffer, sizeof(iobuffer)); // read from device
      run = 1;                                          // started after 1st read
      if(len == 0)
        {
        Sleep(1);                                       // not ready: repeat
        }
      else if(len < 0)                                  // error
        {
        err = 1;
        enablecontrols(1);
        mode = STOP;
        }
      else
        {
        rlen += len;
        if(mode == WRITE)
          {
          WriteFile(file, iobuffer, sizeof(iobuffer), &rwnum, 0);  // write to file
          if(rwnum != sizeof(iobuffer))
            mode = STOP;
          }
        }
      }
    else if(mode == PLAY)                               // PLAY mode
      {
      ReadFile(file, iobuffer, sizeof(iobuffer), &rwnum, 0);    // read from file
      len = rwnum;
      if(len == 0)                                      // end of file
        {
        mode = STOP;
        }
      else
        {
        int nb; char *bp;
        for(bp = iobuffer; len > 0 && mode == PLAY;)
          {
          nb = X16_write(card, bp, len);                // write to device
          run = 1;
          if(nb < len) Sleep(1);
          bp += nb; len -= nb;
          rlen += nb;
          }
        }
      }
    if(mode == STOP)
      {
      if(err)
        {
        SetDlgItemText(MainWin, IDC_MODE, TEXT( "No signal" ));
        err = 0;
        }
      else
        SetDlgItemText(MainWin, IDC_MODE, mnames[mode]);
      if(file) { CloseHandle(file); file = 0; }         // close file
      enablecontrols(1);
      X16_set_mute(card, 1);
      X16_stop(card);
      }
    oldmode = mode;
    }
}


int getfilename(wchar_t *name, wchar_t *title)          // browse file
{
  static OPENFILENAME ofninfo;
  wchar_t ff[] = TEXT( "VideoFiles (*.mpg)\0*.mpg\0AllFiles (*.*)\0*.*\0" );

  ofninfo.lStructSize = sizeof(OPENFILENAME);
  ofninfo.hwndOwner = MainWin;
  ofninfo.hInstance = 0;
  ofninfo.lpstrFilter = ff;
  ofninfo.lpstrCustomFilter = NULL;
  ofninfo.nMaxCustFilter = 0;
  ofninfo.nFilterIndex = 1;
  ofninfo.lpstrFile = name;
  ofninfo.nMaxFile = MAXPATH;
  ofninfo.lpstrFileTitle = NULL;
  ofninfo.nMaxFileTitle = 0;
  ofninfo.lpstrInitialDir = TEXT( ".\\" );
  ofninfo.lpstrTitle = title;
  ofninfo.Flags = OFN_HIDEREADONLY | OFN_PATHMUSTEXIST;
  ofninfo.nFileOffset = 0;
  ofninfo.nFileExtension = 0;
  ofninfo.lpstrDefExt =  TEXT( "*" );
  ofninfo.lCustData = 0L;
  ofninfo.lpfnHook = NULL;
  ofninfo.lpTemplateName = NULL;
  if(!GetOpenFileName(&ofninfo))
    {
    if(GetLastError() != 0)             // 0 value means user selected Cancel
      {
      error(AppName, MB_RETRYCANCEL | MB_ICONERROR,
         TEXT( "File not found..." ));
      return(0);
      }
    return(0);
    }
  wcscpy(name, ofninfo.lpstrFile);
  return(1);
}


int error(wchar_t *hdr, int key, wchar_t *fmt, ... )    // error message
{
  va_list argptr;
  static wchar_t str[MAXPATH + 80];
  int res;

  va_start(argptr, fmt);
  vswprintf(str, fmt, argptr);
  res = MessageBox(0, str, hdr, key);
  va_end(argptr);
  return(res);
}


time_t time_(time_t *t)         // not supported ???
{
  SYSTEMTIME st;
  FILETIME ft;
  LARGE_INTEGER ct;
  time_t res;

  GetSystemTime( &st );
  SystemTimeToFileTime( &st, &ft );
  ct.LowPart = ft.dwLowDateTime;
  ct.HighPart = ft.dwHighDateTime;
  ct.QuadPart /= 10000000;
  ct.QuadPart -= 11644473600;
  res = (time_t)ct.QuadPart;
  if( t )
    *t = res;
  return(res);
}


